# function for relative risk calculation.

rr_calc <- function ( tmean = c(10, 20, 40), 
                      coef_mat = pluck (cluster_fits_doy , "daily_arg",1 , "coeff") ,
                      center_val = 15.6, # default across all clusters
                      degree_val = 2,
                      knot_vec = pluck (cluster_fits_doy , "daily_arg",1 , "knots"), 
                      bound_vec = pluck (cluster_fits_doy , "daily_arg",1 , "bound")
){
  ## This function calculates attributable risk from a vector of temperatures and fitted coefficients (coefficients in vector rofm). Moving BS creation to another function causes class issues.
  # Currently implemented for EACH tmean value.  
  require( dlnm) ; require (splines)
  bs_ce  <- bs(center_val,   degree = degree_val,  knots = knot_vec, Bound=bound_vec, intercept = FALSE)
  bs_var <- suppressWarnings(bs(tmean,    degree = degree_val,  knots = knot_vec, Bound=bound_vec, intercept = FALSE ) ) # suppress repetetive warning regarding temperatures outside of original boundaries
  bs_vc   <- scale(bs_var, center = bs_ce,  scale = FALSE)
 
  RR  <- as.numeric ((1-exp(-bs_vc %*% coef_mat) ) )
  
  return(RR)
}



calc_rr_se <- function ( tmean = seq(-25,42, 1), 
                         coef_mat = pluck(cl4_y3, "cp_out", 1, "cp_0", "coefficients")  ,
                         vcov_mat = pluck(cl4_y3, "cp_out", 1, "cp_0", "vcov") ,
                         center_val = 15.6 , # default across all clusters
                         degree_val = 2,
                         knot_vec =  pluck(cl4_y3, "cp_out", 1,  "knots_1"), 
                         bound_vec = c(-25,42)
){
  ## This function calculates attributable risk and variance from a vector of temperatures and fitted coefficients (coefficients in vector rofm). Moving BS creation to another function causes class issues.
  # Currently implemented for EACH tmean value.  
  require( dlnm) ; require (splines)
  bs_ce  <- bs(center_val,   degree = degree_val,  knots = knot_vec, Bound=bound_vec, intercept = FALSE)
  bs_var <- suppressWarnings(bs(tmean,    degree = degree_val,  knots = knot_vec, Bound=bound_vec, intercept = FALSE ) ) # suppress repetetive warning regarding temperatures outside of original boundaries
  bs_vc   <- scale(bs_var, center = bs_ce,  scale = FALSE)
  
  rr_log    <- as.numeric ( bs_vc %*% coef_mat) 
  rr_log_se <- sqrt(pmax(0, rowSums((bs_vc %*% vcov_mat) * bs_vc))) 
  
  
  RR_an <- 1-exp(-rr_log)
  RR_an_low <- 1-exp(-(rr_log - 1.96 * rr_log_se))
  RR_an_high <- 1-exp(-(rr_log + 1.96 * rr_log_se)) 
  
  tibble(rr_log = rr_log, rr_log_se = rr_log_se, RR = RR_an, RR_lci = RR_an_low, RR_uci = RR_an_high)
  
}
 


calc_rr_avg <- function ( tmean_c = daily_dframe$tmean0, #seq(-20,40, by = 1), 
                          cluster_obj_list = cluster_list,
                          cp_name = "cp_0",
                          knot_name = "knots_1") {
  require( dlnm) ; require (splines); require (tidyr); require(purrr)
# Calculates weighted average (based on se) for attributable risk; does not yet calculate RR_ucl or RR_lcl

if(length(cluster_obj_list)>1){

rr_1 <- calc_rr_se(tmean_c, 
             coef_mat = pluck(cluster_obj_list, 1,  cp_name, "coefficients")  ,
             vcov_mat = pluck(cluster_obj_list, 1, cp_name, "vcov") ,
             center_val = 15.6 , # default across all clusters
             degree_val = 2,
             knot_vec =  pluck(cluster_obj_list, 1,  knot_name), 
             bound_vec = range(pluck(cluster_obj_list, 1,  cp_name,  "predvar")))
rr_1$row_id = seq(1:nrow(rr_1)) 

rr_2 <- calc_rr_se(tmean_c, 
                   coef_mat = pluck(cluster_obj_list, 2, cp_name, "coefficients")  ,
                   vcov_mat = pluck(cluster_obj_list, 2, cp_name, "vcov") ,
                   center_val = 15.6 , # default across all clusters
                   degree_val = 2,
                   knot_vec =  pluck(cluster_obj_list, 2,  knot_name), 
                   bound_vec = range(pluck(cluster_obj_list, 2,   cp_name,  "predvar")))
rr_2$row_id = seq(1:nrow(rr_2)) 

RR_an_wt_avg <- bind_rows(rr_1, rr_2, .id = "id"  ) %>% 
  group_by(row_id) %>%
  mutate(wt = rr_log_se/sum(rr_log_se), 
         wt = if_else(!is.nan(wt), wt, 1)) %>%
  summarise(  wt_rr_log = weighted.mean(rr_log, wt))  %>% 
  mutate( RR_an_wt_avg = 1-exp(-wt_rr_log) ) %>% 
  pull(RR_an_wt_avg) 
return(RR_an_wt_avg)
} else{
  rr_calc( tmean =tmean_c, 
           coef_mat = pluck(cluster_obj_list, 1, cp_name, "coefficients") ,
           center_val = 15.6, # default across all clusters
           degree_val = 2,
           knot_vec =  pluck(cluster_obj_list, 1,  knot_name), 
           bound_vec = range(pluck(cluster_obj_list, 1,   cp_name,  "predvar")))
  }
  
}
  
     

cp_rr_calc <- function (  tmean_c = daily_dframe$tmean0, #seq(-20,40, by = 1), 
                          cluster_obj_list = cluster_list,
                          cp_name = "cp_0",
                          knot_name = "knots_1"
                          
){
  ## This function calculates relative risk from a vector of temperatures and fitted coefficients (coefficients in vector rofm). Moving BS creation to another function causes class issues.
  # Currently implemented for EACH tmean value.  
  require( dlnm) ; require (splines)
  coef_mat =  pluck(cluster_obj_list, 1,  cp_name, "coefficients") 
  knot_vec =  pluck(cluster_obj_list, 1,  knot_name)
  bound_vec = range(pluck(cluster_obj_list, 1,   cp_name,  "predvar"))
 
  bs_ce  <- bs(15.6,   degree = 2,  knots = knot_vec, Bound=bound_vec, intercept = FALSE) 
  bs_var <- suppressWarnings(bs(tmean_c,    degree = 2,  knots = knot_vec, Bound=bound_vec, intercept = FALSE ) ) # suppress repetetive warning regarding temperatures outside of original boundaries
  bs_vc   <- scale(bs_var, center = bs_ce,  scale = FALSE) 

  rr_log    <- as.numeric ( bs_vc %*% coef_mat)  
  
  RR  <- exp( rr_log)

  
  return(RR)
}

  